function pdptool(varargin)
% PDPTOOL Parallel Distributed Processing application for simulating 
%         neural network models, as discussed in Parallel distributed 
%         processing: Explorations in the microstructure of cognition
%         (McClelland, J. L., Rumelhart,D. E., and the PDP research group.
%         (1986)).
%         
%      
%      Syntax : 
%      (a) pdptool
%      (b) pdptool <network script(s)> 
%      (c) pdptool nogui <network script | matlab script>
%   
%      Description :
%      (a) pdptool by itself starts the application in gui mode. It opens 
%          the main pdp window which allows user to create ,load and run
%          neural network simulations. 
%      (b) pdptool with one or more network script file names (.net) starts
%          the application in gui mode. It loads all the networks defined 
%          in each of the .net files. The last loaded network will be the
%          current network available as a 'net' struct variable in the 
%          MATLAB workspace.The networks are identified by their names and
%          listed in the network pop-up menu of the pdp window.The current
%          network can be changed by selecting from the list of available 
%          networks.
%      (c) pdptool with nogui option starts application without a graphical
%          interface. All network actions would have to be performed 
%          subsequently by invoking specific commands typed on the MATLAB 
%          command line one after the other.Alternatively, a MATLAB 
%          script(.m) file, containing all  commands can be passed as   
%          argument to the pdptool command. pdptool nogui can also be  
%          called with the name of a single network script file. This loads  
%          the network defined in the file just like in (b) ,except there 
%          are no graphical features.
%

global PDPAppdata net;
gui  = 1;
pdpargs =[];
if nargin > 0
   argchk = strcmpi(varargin,'nogui');
   if any(argchk)
      gui = 0;
   end
   pdpargs = varargin(~argchk);
end
initsystem(gui);
if gui && isempty(pdpargs)
   pdp;
end
if ~isempty(pdpargs)
   for i = 1: numel(pdpargs)
       if exist(pdpargs{i},'file') ~=2
          error('File %s cannot be found', pdpargs{i});
       end
       [p,n,e] = fileparts(pdpargs{i});
       if ~any(strcmpi(e,{'.net','.m'}))
          error('Invalid file format - %s : should be .net or .m file',...
                pdpargs{i});
       end
       if strcmpi(e,'.net')
          pdp;
          loadscript(pdpargs{i});
       else
          PDPAppdata.lognetwork = 1;
          fr = textread(pdpargs{i},'%s','delimiter','\n');
          cellfun(@eval,fr);
       end
   end    
end
if isdeployed
  command = '';
  fprintf(1,'\n\nEnter pdpquit to quit\n\n');
  while ~strcmpi(command,'pdpquit')
      try
      command = input('>>pdp ','s');
      if exist('PDPAppdata','var') ~= 1
         fprintf(1,'\nPdptool is not available anymore. Please start a new session to continue\n');
         break;
      else
          T= evalc(command);
          if ~isempty(T)
             fprintf(1,'%s \n',T);
          end
          if ~strcmpi(command,'pdpquit') && ~isempty(regexpi(T,'error'))
             updatelog(command);
          end
      end
      catch ME1
          e = ME1.identifier;
          fprintf(1,'Error using %s\n%s\n',command,e);
%           lasterror('reset');
      end
  end
end
   

